/*

  Copyright (C) 1999 President and Fellows of Harvard College

  All rights reserved.

  NOTICE: This software is provided "as is", without any warranty,
  including any implied warranty for merchantability or fitness for a
  particular purpose.  Under no circumstances shall Harvard University
  or its faculty, staff, students or agents be liable for any use of,
  misuse of, or inability to use this software, including incidental
  and consequential damages.

  License is hereby given to use, modify, and redistribute this
  software, in whole or in part, for any commercial or non-commercial
  purpose, provided that the user agrees to the terms of this
  copyright notice, including disclaimer of warranty, and provided
  that this copyright notice, including disclaimer of warranty, is
  preserved in the source code and documentation of anything derived
  from this software.  Any redistributor of this software or anything
  derived from this software assumes responsibility for ensuring that
  any parties to whom wuch a redistribution is made are fully aware of
  the terms of this license and disclaimer.

  Author: Brad Karp, Harvard University EECS, September, 1997

*/

/* main.c -- argument processing and glue for HUMR, Harvard User-Level
   Metricom Radio driver */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/sockio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>
#include "debug.h"
#include "humr.h"
#include "arp.h"
#include "modem.h"
#include "loop.h"

/* mask of which events to report for debugging */
int dmask = 0;

/* before running, /dev/tun<n> must be ifconfig'ed with src and dst addrs
   and netmask, and routes should be installed appropriately. */
int main(int argc, char **argv)
{
  int ochar, s, i;
  char *tunname;
  struct ifreq thereq;

  if (argc < 5) {
    fprintf(stderr,
	    "usage: %s -s <radio tty> -t <tun device> [-d <debugmask>] ",
	    argv[0]);
    fprintf(stderr, "[-f <fallback IP>]\n");
    exit(1);
  }
  while ((ochar = getopt(argc, argv, "s:t:d:f:li:")) != EOF) {
    switch (ochar) {
    case 'd':
      /* debug mask */
      dmask = atoi(optarg);
      break;
    case 'f':
      /* fallback IP next hop */
      inet_aton(optarg, &fallback);
      break;
    case 's':
      /* tty for radio */
      if ((modemfd = open(optarg, O_RDWR, 0)) < 0) {
	fprintf(stderr, "can't open %s\n", optarg);
	exit(1);
      }
      break;
    case 't':
      /* tun device to kernel */
      if ((tunnelfd = open(optarg, O_RDWR, 0)) < 0) {
	fprintf(stderr, "can't open %s\n", optarg);
	exit(1);
      }
      tunname = optarg;
      break;
    case 'l':
      /* IP lookups on all forwarded packets */
      lflag = 1;
      break;
    case 'i':
      /* don't ARP for named destination */
      inet_aton(optarg, &ignore);
      break;
    default:
      break;
    }
  }
  /* figure out local IP address of tunnel */
  /* XXX -- requires tunnel dev btwn 0 and 9 */
  strcpy((char *)&thereq.ifr_name, "tunX");
  thereq.ifr_name[3] = tunname[strlen(tunname)-1];
  s = socket(PF_INET, SOCK_DGRAM, 0);
  ioctl(s, SIOCGIFADDR, &thereq);
  myip = ((struct sockaddr_in *)&thereq.ifr_addr)->sin_addr;
  fprintf(stderr, "tunnel local addr is %s\n", inet_ntoa(myip));
  /* set tunnel interface UP */
  ioctl(s, SIOCGIFFLAGS, &thereq);
  thereq.ifr_flags |= IFF_UP;
  ioctl(s, SIOCSIFFLAGS, &thereq);
  /* initialize regexps */
  modem_regex_init();
  /* initialize NNARP table */
  nnarp_init();
  /* initialize modem */
  modem_reset(modemfd);
  /* initialize kernel routing socket */
  if (lflag) {
    rtfd_init();
  }
  /* initialize callouts (call once, set prev callout time) */
  gettimeofday(&lastcout, NULL);
  keepalive = lastcout;
  timertick();
  ask_neighbors();
  mainloop();
}
