/*

  Copyright (C) 1999 President and Fellows of Harvard College

  All rights reserved.

  NOTICE: This software is provided "as is", without any warranty,
  including any implied warranty for merchantability or fitness for a
  particular purpose.  Under no circumstances shall Harvard University
  or its faculty, staff, students or agents be liable for any use of,
  misuse of, or inability to use this software, including incidental
  and consequential damages.

  License is hereby given to use, modify, and redistribute this
  software, in whole or in part, for any commercial or non-commercial
  purpose, provided that the user agrees to the terms of this
  copyright notice, including disclaimer of warranty, and provided
  that this copyright notice, including disclaimer of warranty, is
  preserved in the source code and documentation of anything derived
  from this software.  Any redistributor of this software or anything
  derived from this software assumes responsibility for ensuring that
  any parties to whom wuch a redistribution is made are fully aware of
  the terms of this license and disclaimer.

  Author: Brad Karp, Harvard University EECS, September, 1997

*/

/* Ricochet-specific lengths */
#define MAX_HANDLE_LEN 18
#define IDENT_LEN 4
#define RICOCHET_MTU 1152             /* Cheshire's recommendation; NFS?? */
#define MAX_STAR_PLEN (RICOCHET_MTU+MAX_HANDLE_LEN+3)

/* Ricochet packet types */
#define RICOCHET_PTYPE_NULL 0        /* zero-length (null) packet */
#define RICOCHET_PTYPE_ATR 1         /* response to an AT command */
#define RICOCHET_PTYPE_ERR 2         /* error message */
#define RICOCHET_PTYPE_INFO 3        /* info message */
#define RICOCHET_PTYPE_ACK 4         /* packet ACK */
#define RICOCHET_PTYPE_RADIO 5       /* from a radio */
#define RICOCHET_PTYPE_HUIP 6        /* HUMR IP packet */
#define RICOCHET_PTYPE_IAM 7         /* "I-am" packet */

#define min(x,y) (((x) < (y)) ? (x) : (y))

#define CHECKSUM_INVALID 0xdeadbeef

/* XXX -- hack: the fd of the modem device */
extern int modemfd;

/* initialize regexes for modem manipulation */
void modem_regex_init(void);
/* reset the modem; put it into a known state where we can issue commands */
void modem_reset(int fd);
/* get neighbor (handle) list from modem */
void ask_neighbors(void);
/* classify type of starmode packet */
int packet_type(u_char *buf, int len, u_char *psrc);
/* extract contents of modem command reply */
int process_packet_ATR(u_char *buf);
/* extract contents of info message */
int process_packet_INFO(u_char *buf);
/* extract contents of err message */
int process_packet_ERR(u_char *buf);
/* compute checksum over the packet */
int packet_checksum(u_char *buf, int len);
/* is NUL-terminated string argument a handle? */
int ishandle(char *addr);
